/*
(c) 2015 Tuomas Airaksinen

This file is part of automate-webui.

automate-webui is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

automate-webui is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with automate-webui.  If not, see <http://www.gnu.org/licenses/>.
*/

body {
    padding-top: 50px;
}

.starter-template {
    padding-bottom: 20px;
}

.alarmbox {
    color: red;
}

.messages {
    color: red;
}

/* Login */

.asteriskField {
    display: none;
}

/* Object listings*/

.object_list_column {
    max-width: 600px;
    margin-left: auto;
    margin-right: auto;
}

.object_status_column, .program_active_column, .program_priority_column{
    text-align: right;
}

.group_title {
    font-size: 160%;
    padding-bottom: 5pt;
    padding-top: 5pt;
    font-weight: bold;
    background: rgba(49, 139, 254, 0.19);
}

.group_content {
    padding-top: 10pt;
    padding-bottom: 10pt;
}

.object_type_title {
    font-size: 120%;
    padding-bottom: 5pt;
    padding-top: 5pt;
    font-weight: bold;
    background: rgba(49, 139, 254, 0.19);
}

em.status_active {
    color: green;
}

em.status_inactive {
    color: red;

}
em.status_changing {
    color: orange;
}

em.condition_active {
    color: green;
    font-style: normal;
}

em.condition_inactive {
    color: red;
    font-style: normal;
}

.title_row {
    font-weight: bold;
}

.info_panel {
}

div.object_row {
    padding-top: 8pt;
    padding-bottom: 8pt;
}

hr.object_row {
    margin: 2pt;
}

/* Console view */

pre.log {
    font-size: 80%;
    background: #464646;
    height: 40em;
    overflow: scroll;
}

.copyright { padding-top: 2em; text-align: center }
.ansi_fore { color: #FFFFFF; }
.ansi_back { background-color: #000000; }
.ansi1 { font-weight: bold; }
.ansi3 { font-style: italic; }
.ansi4 { text-decoration: underline; }
.ansi9 { text-decoration: line-through; }
.ansi30 { color: #000000; }
.ansi31 { color: #FF0000; }
.ansi32 { color: #00FF00; }
.ansi33 { color: #FFFF00; }
.ansi34 { color: #0000FF; }
.ansi35 { color: #FF00FF; }
.ansi36 { color: #00FFFF; }
.ansi37 { color: #FFFFFF; }
.ansi40 { background-color: #000000; }
.ansi41 { background-color: #FF0000; }
.ansi42 { background-color: #00FF00; }
.ansi43 { background-color: #FFFF00; }
.ansi44 { background-color: #0000FF; }
.ansi45 { background-color: #FF00FF; }
.ansi46 { background-color: #00FFFF; }
.ansi47 { background-color: #FFFFFF; }

/* PlantUML view */

img.puml {
    max-width: 100%;
    height: auto;
}