{% load automate_tags %}
{% mocked_services as mocked_services_ %}
{% if mocked_services_%}
  <br>
  <div class="panel panel-danger">
    <div class="panel-heading">
      <h4>Mocked services</h4>
    </div>
    <div class="panel-body">
      <ul>
        {% for name, service in mocked_services_ %}
          <li>{{name}} <a href="{% url "reload_service" service.id %}">(reload)</a>
        {% endfor %}
      </ul>
    </div>
  </div>
{% endif %}
