<input
        class='sliderfields slider_sensor_{{ field.field.sensor_name }}'
        type="text"
        name='status'
        data-slider-id='slider_{{ field.field.sensor_name }}'
        data-slider-min="{{ field.field.min_value }}"
        data-slider-max="{{ field.field.max_value }}"
        data-slider-step="{{ field.field.step }}"
        data-slider-value="{{ field.value }}"/>
