{% load crispy_forms_tags staticfiles %}
<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="utf-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <meta name="description" content="">
  <meta name="author" content="">

  <title>{{ system.name }}</title>

  <link href="{% static "bootstrap/css/bootstrap.min.css" %}" rel="stylesheet">
  <script src="{% static "jquery.min.js" %}"></script>
  <script src="{% static "bootstrap/js/bootstrap.min.js" %}"></script>
  <link href="{% static "style.css" %}" rel="stylesheet">
  <link href="{% static "login.css" %}" rel="stylesheet">
  <script>
    var source = "{{source}}";
  </script>
</head>

<body>


<div class="starter-template">
  <div class="container">

    {% if messages %}
      <h2 class="messages">Messages from the system</h2>
      <ul class="messages">
        {% for message in messages %}
          <li{% if message.tags %} class="messages {{ message.tags }}"{% endif %}>{{ message }}</li>
        {% endfor %}
      </ul>
    {% endif %}

    {% block content %}

      <div class="container">
        <h2 class="form-signin" role="form">Login to {{ system.name }}</h2>
        {% crispy form %}
      </div>
    {% endblock %}
  </div>
</div>
</body>
</html>
