{% extends "base.html" %}
{% load automate_tags %}
{% block title %}{{ name }}{% endblock %}
{% block jsscripts %}{% endblock %}
{% block content %}
  {% block info %}
    <h1>Status of {{ name }} set to {{ status }}</h1>
  {% endblock %}
  <br>
  <a class="btn btn-warning" href="{% url 'toggle_value' name %}">Toggle value</a>
{% endblock %}

